<?php

/**
 * Class ControllerExtensionPaymentVtbPay
 * This class controls the VTB payment extension. 
 * It includes methods for rendering the payment settings page, 
 * processing the form submissions, validating the inputs, etc.
 */
class ControllerExtensionPaymentVtbPay extends Controller 
{
    private $error = [];
    
    
    /**
     * getFormValue method
     *
     * This method fetches the form value from the request or the configuration.
     * It checks if a particular key is set in the request, 
     * if it's set then it returns the corresponding value,
     * otherwise, it fetches the value from the config.
     *
     * @param array $request Array containing form data.
     * @param object $config OpenCart Config object.
     * @param string $key Key to look for in the request and config.
     * @return string The value of the provided key.
     */
    private function getFormValue($request, $config, $key) 
    {
        return trim($request[$key] ?? $config->get($key));
    }
    
    
    /**
     * getCheckboxValue method
     *
     * This method fetches the checkbox value from the request or the configuration.
     * It fetches the form value and checks if it's truthy,
     * then it returns '1', otherwise, it returns '0'.
     *
     * @param array $request Array containing form data.
     * @param object $config OpenCart Config object.
     * @param string $key Key to look for in the request and config.
     * @return string The value ('1' or '0') of the provided key.
     */
    private function getCheckboxValue($request, $config, $key) 
    {
        return $this->getFormValue($request, $config, $key) ? '1' : '0';
    }


    /**
     * index method
     *
     * This is the default method for the controller.
     * It loads the language file, the setting model, the order status model, 
     * handles the form submission, and prepares the data to render the settings page.
     *
     * @return void
     */
    public function index() 
    {
        $this->load->language('extension/payment/vtbpay');
        $this->document->setTitle($this->language->get('heading_title'));
        $this->load->model('setting/setting');
        $this->load->model('localisation/order_status');

        // If the request is POST and validation is passed, then save the changes
        if (
            ($this->request->server['REQUEST_METHOD'] == 'POST') && 
            $this->validate()
        ) {
            $this->model_setting_setting->editSetting('payment_vtbpay', $this->request->post);
            $this->session->data['success'] = $this->language->get('text_success');
            $this->response->redirect($this->url->link(
                'extension/payment/vtbpay', 
                'user_token=' . $this->session->data['user_token'] . '&type=payment', 
                true
            ));
        }
        
        // set value for form fields
        $fields = [
            'desc',
            'client_id',
            'client_secret',
            'merchant_authorization',
            'transaction_end_status',
            'test_mode',
            'logging',
            'status',
            'sort_order'
        ];
        
        $checkboxFields = [
            'test_mode',
            'logging',
            'status'
        ];
        
        foreach ($fields as $field) {
            if (in_array($field, $checkboxFields)) {
                $data['payment_vtbpay_' . $field] = $this->getCheckboxValue(
                    $this->request->post, 
                    $this->config, 
                    'payment_vtbpay_' . $field
                );
            } 
            else {
                $data['payment_vtbpay_' . $field] = $this->getFormValue(
                    $this->request->post, 
                    $this->config, 
                    'payment_vtbpay_' . $field
                );
            }
        }
        
        if (empty($data['payment_vtbpay_desc'])) {
            $data['payment_vtbpay_desc'] = $this->language->get('default_value_desc');
        }

        $data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();
        
        $this->showPaymentSettingsPage($data);
    }
    
  
    /**
     * showPaymentSettingsPage method
     *
     * This method prepares the breadcrumbs and other necessary data for the view,
     * and then it renders the payment settings page.
     *
     * @param array $data The data array that will be passed to the view.
     * @return void
     */
    private function showPaymentSettingsPage($data) 
    {
        $data['breadcrumbs'] = [];

        $data['breadcrumbs'][] = [
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true)
        ];

        $data['breadcrumbs'][] = [
            'text' => $this->language->get('text_extension'),
            'href' => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true)
        ];

        $data['breadcrumbs'][] = [
            'text' => $this->language->get('heading_title'),
            'href' => $this->url->link('extension/payment/vtbpay', 'user_token=' . $this->session->data['user_token'], true)
        ];

        $data['action'] = $this->url->link('extension/payment/vtbpay', 'user_token=' . $this->session->data['user_token'], true);
        $data['cancel'] = $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true);
        
        $data['header'] = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer'] = $this->load->controller('common/footer');
        
        $data = $this->setAlert($data);

        // Load view template
        $this->response->setOutput($this->load->view('extension/payment/vtbpay', $data));
    }
    
    
    /**
     * setAlert method
     *
     * Method that sets the display of alerts: 
     * error messages and a message about successful saving
     *
     * @param array $data Array containing Output data.
     */
    protected function setAlert($data)
    {
        if (empty($this->error)) {
            // successful save output
            if (isset($this->session->data['success'])) {
                $data['success'] = $this->session->data['success'];
                unset($this->session->data['success']);
            }
        }
        else {
            $data['error'] = $this->error;
        }
        
        return $data;
    }
    

    /**
     * validate method
     *
     * This method validates the form submission.
     * It checks if the user has the permission to modify the settings,
     * validates the necessary fields
     *
     * @return bool Returns true if no validation error, otherwise returns false.
     */
    protected function validate() 
    {
        if (!$this->user->hasPermission('modify', 'extension/payment/vtbpay')) {
            $this->error['warning'] = $this->language->get('error_msg_permission');
        }
        
        $required_fields = [
            'client_id',
            'client_secret'
        ];
        
        foreach ($required_fields as $required_field) {
            if (!$this->request->post['payment_vtbpay_' . $required_field]) {
                $this->error[$required_field] = true;
            }
        }

        // Return true if no error, false otherwise
        return empty($this->error);
    }
}
